@@ -1,18 +1,18 @@
 from typing import Optional
 
+import re
 import urllib3
 import threading
 import logging
-import sys
 import json
 import asyncio
+from hashlib import sha256
 from typing import List
 from featurehub_sdk.edge_service import EdgeService
 from featurehub_sdk.featurehub_repository import FeatureHubRepository
 from featurehub_sdk.version import sdk_version
 
-log = logging.getLogger(sys.modules[__name__].__name__)
-
+log = logging.getLogger('featurehub_sdk')
 
 class PollingEdgeService(EdgeService):
     _interval: int
@@ -20,7 +20,11 @@ class PollingEdgeService(EdgeService):
     _cancel: bool
     _thread: Optional[threading.Timer]
     _client_eval: bool
+    _stopped: bool
     _http: urllib3.PoolManager
+    _cache_control_pattern: re.Pattern
+    _sha_context: Optional[str]
+    _etag: Optional[str]
 
     def __init__(self, edge_url: str, api_keys: List[str],
                  repository: FeatureHubRepository,
@@ -28,10 +32,14 @@ class PollingEdgeService(EdgeService):
         self._interval = interval
         self._repository = repository
         self._cancel = False
+        self._stopped = False
         self._thread = None
         self._client_eval = '*' in api_keys[0]
         self._context = None
+        self._etag = None
+        self._sha_context = None
         self._http = urllib3.PoolManager()
+        self._cache_control_pattern = re.compile('max-age=(\\d+)')
 
         self._url = f"{edge_url}features?" + "&".join(map(lambda i: 'apiKey=' + i, api_keys))
         log.debug(f"polling url {self._url}")
@@ -47,13 +55,36 @@ class PollingEdgeService(EdgeService):
     # this does the business, calls the remote service and gets the features back
     async def _get_updates(self):
         # TODO: set timeout of tcp requests to 12 seconds, or give users control over it using environ vars
-        url = self._url if self._context is None else f"{self._url}&{self._context}"
-        log.log(5, "polling ", url)
-        resp = self._http.request(method='GET', url=url, headers={'X-SDK': 'Python', 'X-SDK-Version': sdk_version})
-        log.log(5, "polling status", resp.status)
+        sha_context = "0" if self._sha_context is None else self._sha_context
+        url = f"{self._url}&contextSha={sha_context}"
+
+        log.debug("polling %s", url)
+        headers = {
+            'X-SDK': 'Python',
+            'X-SDK-Version': sdk_version
+        }
+
+        if self._etag:
+            headers['if-none-match'] = self._etag
+
+        if self._context:
+            headers['x-featurehub'] = self._context
+
+        resp = self._http.request(method='GET', url=url, headers=headers)
+        log.debug("polling status %s", resp.status)
+
+        if resp.status == 200 or resp.status == 236:
+            if 'etag' in resp.headers:
+                self._etag = resp.headers['etag']
+
+            if 'cache-control' in resp.headers:
+                self._cache_control_polling_interval(resp.headers['cache-control'])
 
-        if resp.status == 200:
             self._process_successful_results(json.loads(resp.data.decode('utf-8')))
+
+            # if it is a 236, we have been told to stop
+            if resp.status == 236:
+                self._stopped = True
         elif resp.status == 404: # no such key
             self._repository.notify("failed", None)
             self._cancel = True
@@ -63,12 +94,19 @@ class PollingEdgeService(EdgeService):
             return
         # otherwise its likely a transient failure, so keep trying
 
+    def _cache_control_polling_interval(self, cache_control: str):
+        max_age = re.findall(self._cache_control_pattern, cache_control)
+        if max_age: # not none and not empty
+            new_interval = int(max_age[0])
+            if new_interval > 0:
+                self._interval = new_interval
+
     # this is essentially a repeating task because it "calls itself"
     # another way to do this is with a separate class that is itself a thread descendant
     # which waits for the requisite time,  then triggers a callback and then essentially does the same thing
     # if we need a repeating task elsewhere, we should consider refactoring this
     async def poll_with_interval(self):
-        if not self._cancel:
+        if not self._cancel and not self._stopped:
             await self._get_updates()
             if not self._cancel and self._interval > 0:
                 self._thread = threading.Timer(self._interval, self.poll_again)
@@ -97,6 +135,7 @@ class PollingEdgeService(EdgeService):
     async def context_change(self, header: str):
         old_context = self._context
         self._context = header
+        self._sha_context = sha256(header.encode('utf-8')).hexdigest()
         if old_context != header:
             await self._get_updates()
 
@@ -104,10 +143,18 @@ class PollingEdgeService(EdgeService):
     def cancelled(self):
         return self._cancel
 
+    @property
+    def stopped(self):
+        return self._stopped
+
+    @property
+    def interval(self):
+        return self._interval
+
     # we get returned a bunch of environments for a GET/Poll API so we need to cycle through them
     # the result is different for streaming
     def _process_successful_results(self, data):
-        log.log(5, "featurehub polling data was %s", data)
+        log.debug("featurehub polling data was %s", data)
         for feature_apikey in data:
             if feature_apikey:
                 self._repository.notify("features", feature_apikey['features'])
